@@ -67,14 +67,14 @@ impl JsonValue {
         }
     }
 
-    pub fn elements<'a>(self, json: &'a Json) -> JsonElementIter<'a> {
+    pub fn elements(self, json: &Json) -> JsonElementIter {
         match self {
             JsonValue::Array(array) => array.elements(json),
             _ => JsonElementIter { json, next: 0 },
         }
     }
 
-    pub fn members<'a>(self, json: &'a Json) -> JsonMemberIter<'a> {
+    pub fn members(self, json: &Json) -> JsonMemberIter {
         match self {
             JsonValue::Object(object) => object.members(json),
             _ => JsonMemberIter { json, next: 0 },
@@ -145,7 +145,7 @@ pub struct JsonString {
 }
 impl JsonString {
     pub fn as_str<'a>(&self, json: &'a Json) -> &'a str {
-        &json.strings[(self.start as usize)..(self.end as usize)]
+        &json.strings[self.start as usize..self.end as usize]
     }
 }
 
@@ -155,7 +155,7 @@ pub struct JsonArray {
     last: u32,
 }
 impl JsonArray {
-    pub fn elements<'a>(self, json: &'a Json) -> JsonElementIter<'a> {
+    pub fn elements(self, json: &Json) -> JsonElementIter {
         JsonElementIter {
             json,
             next: self.first as _,
@@ -195,7 +195,7 @@ impl<'json> FromJson<'json> for JsonKey {
     fn from_json(value: JsonValue, _: &'json Json) -> Result<Self, JsonConvertError> {
         match value {
             JsonValue::Str(s) => Ok(JsonKey::Str(s)),
-            JsonValue::String(s) => Ok(JsonKey::String(s.clone())),
+            JsonValue::String(s) => Ok(JsonKey::String(s)),
             _ => Err(JsonConvertError),
         }
     }
@@ -226,7 +226,7 @@ impl JsonObject {
         JsonValue::Null
     }
 
-    pub fn members<'a>(self, json: &'a Json) -> JsonMemberIter<'a> {
+    pub fn members(self, json: &Json) -> JsonMemberIter {
         JsonMemberIter {
             json,
             next: self.first as _,
@@ -272,7 +272,7 @@ impl<'a> Iterator for JsonElementIter<'a> {
         if self.next != 0 {
             let element = &self.json.elements[self.next];
             self.next = element.next as usize;
-            Some(clone_json_value(&element.value))
+            Some(element.value.clone())
         } else {
             None
         }
@@ -291,38 +291,13 @@ impl<'a> Iterator for JsonMemberIter<'a> {
         if self.next != 0 {
             let member = &self.json.members[self.next];
             self.next = member.next as usize;
-            Some((
-                member.key.as_str(self.json),
-                clone_json_value(&member.value),
-            ))
+            Some((member.key.as_str(self.json), member.value.clone()))
         } else {
             None
         }
     }
 }
 
-fn clone_json_value(value: &JsonValue) -> JsonValue {
-    match value {
-        JsonValue::Null => JsonValue::Null,
-        JsonValue::Boolean(b) => JsonValue::Boolean(*b),
-        JsonValue::Integer(i) => JsonValue::Integer(*i),
-        JsonValue::Number(n) => JsonValue::Number(*n),
-        JsonValue::Str(s) => JsonValue::Str(*s),
-        JsonValue::String(s) => JsonValue::String(JsonString {
-            start: s.start,
-            end: s.end,
-        }),
-        JsonValue::Array(a) => JsonValue::Array(JsonArray {
-            first: a.first,
-            last: a.last,
-        }),
-        JsonValue::Object(a) => JsonValue::Object(JsonObject {
-            first: a.first,
-            last: a.last,
-        }),
-    }
-}
-
 pub struct Json {
     strings: String,
     elements: Vec<JsonArrayElement>,
@@ -386,7 +361,7 @@ impl Json {
             R: io::BufRead,
         {
             let buf = reader.fill_buf()?;
-            if buf.len() > 0 && buf[0] == byte {
+            if !buf.is_empty() && buf[0] == byte {
                 reader.consume(1);
                 Ok(true)
             } else {
@@ -403,8 +378,7 @@ impl Json {
                 match buf
                     .iter()
                     .enumerate()
-                    .skip_while(|(_, c)| c.is_ascii_whitespace())
-                    .next()
+                    .find(|(_, c)| !c.is_ascii_whitespace())
                 {
                     Some((0, _)) | None => return Ok(()),
                     Some((i, _)) => reader.consume(i),
@@ -547,7 +521,7 @@ impl Json {
                         R: io::BufRead,
                     {
                         let buf = reader.fill_buf()?;
-                        if buf.len() > 0 {
+                        if !buf.is_empty() {
                             let byte = buf[0];
                             if byte.is_ascii_digit() {
                                 reader.consume(1);
@@ -613,22 +587,28 @@ impl Json {
         read_value(self, reader)
     }
 
-    pub fn write(&self, buf: &mut Vec<u8>, value: &JsonValue) {
-        fn append_str(buf: &mut Vec<u8>, s: &str) {
-            buf.push(b'"');
+    pub fn write<W>(&self, buf: &mut W, value: &JsonValue) -> io::Result<()>
+    where
+        W: io::Write,
+    {
+        fn append_str<W>(buf: &mut W, s: &str) -> io::Result<()>
+        where
+            W: io::Write,
+        {
+            buf.write_all(b"\"")?;
             for c in s.chars() {
                 match c {
-                    '\"' => buf.extend_from_slice(b"\\\""),
-                    '\\' => buf.extend_from_slice(b"\\\\"),
-                    '\x08' => buf.extend_from_slice(b"\\b"),
-                    '\x0c' => buf.extend_from_slice(b"\\f"),
-                    '\n' => buf.extend_from_slice(b"\\n"),
-                    '\r' => buf.extend_from_slice(b"\\r"),
-                    '\t' => buf.extend_from_slice(b"\\t"),
+                    '\"' => buf.write_all(b"\\\"")?,
+                    '\\' => buf.write_all(b"\\\\")?,
+                    '\x08' => buf.write_all(b"\\b")?,
+                    '\x0c' => buf.write_all(b"\\f")?,
+                    '\n' => buf.write_all(b"\\n")?,
+                    '\r' => buf.write_all(b"\\r")?,
+                    '\t' => buf.write_all(b"\\t")?,
                     _ => {
                         let c = c as u32;
                         if c >= 32 && c <= 126 {
-                            buf.push(c as u8);
+                            buf.write_all(&[c as _])?;
                         } else {
                             fn to_hex_digit(n: u32) -> u8 {
                                 let n = (n & 0xf) as u8;
@@ -639,74 +619,69 @@ impl Json {
                                 }
                             }
 
-                            buf.extend_from_slice(b"\\u");
+                            buf.write_all(b"\\u")?;
                             let c = c.to_le();
-                            buf.extend_from_slice(&[
+                            buf.write_all(&[
                                 to_hex_digit(c >> 12),
                                 to_hex_digit(c >> 8),
                                 to_hex_digit(c >> 4),
                                 to_hex_digit(c),
-                            ]);
+                            ])?;
                         }
                     }
                 }
             }
-            buf.push(b'"');
+            buf.write_all(b"\"")?;
+            Ok(())
         }
 
         match value {
-            JsonValue::Null => {
-                buf.extend_from_slice(b"null");
-            }
-            JsonValue::Boolean(true) => {
-                buf.extend_from_slice(b"true");
-            }
-            JsonValue::Boolean(false) => {
-                buf.extend_from_slice(b"false");
-            }
+            JsonValue::Null => buf.write_all(b"null"),
+            JsonValue::Boolean(true) => buf.write_all(b"true"),
+            JsonValue::Boolean(false) => buf.write_all(b"false"),
             JsonValue::Integer(i) => {
-                use io::Write;
-                let _ = write!(buf, "{}", i);
+                write!(buf, "{}", i)
             }
             JsonValue::Number(n) => {
-                use io::Write;
-                let _ = write!(buf, "{}", n);
+                write!(buf, "{}", n)
             }
             JsonValue::Str(s) => append_str(buf, s),
             JsonValue::String(s) => append_str(buf, s.as_str(self)),
             JsonValue::Array(a) => {
-                buf.push(b'[');
+                buf.write_all(b"[")?;
                 let mut next = a.first as usize;
                 if next != 0 {
                     loop {
                         let element = &self.elements[next];
-                        self.write(buf, &element.value);
+                        self.write(buf, &element.value)?;
                         next = element.next as _;
                         if next == 0 {
                             break;
                         }
-                        buf.push(b',');
+                        buf.write_all(b",")?;
                     }
                 }
-                buf.push(b']');
+                buf.write_all(b"]")?;
+                Ok(())
             }
             JsonValue::Object(o) => {
-                buf.push(b'{');
+                buf.write_all(b"{")?;
                 let mut next = o.first as usize;
                 if next != 0 {
                     loop {
                         let member = &self.members[next];
-                        append_str(buf, member.key.as_str(self));
-                        buf.push(b':');
-                        self.write(buf, &member.value);
+                        append_str(buf, member.key.as_str(self))?;
+                        buf.write_all(b":")?;
+                        self.write(buf, &member.value)?;
                         next = member.next as _;
                         if next == 0 {
                             break;
                         }
-                        buf.push(b',');
+                        buf.write_all(b",")?;
                     }
                 }
-                buf.push(b'}');
+                buf.write_all(b"}")?;
+                Ok(())
             }
         }
     }
@@ -726,7 +701,7 @@ mod tests {
     fn write_value() {
         fn assert_json(buf: &mut Vec<u8>, expected: &str, value: JsonValue, json: &mut Json) {
             buf.clear();
-            json.write(buf, &value);
+            json.write(buf, &value).unwrap();
             assert_eq!(expected, std::str::from_utf8(buf).unwrap());
         }
 
@@ -786,7 +761,7 @@ mod tests {
 
         let mut buf = Vec::new();
         let array = array.into();
-        json.write(&mut buf, &array);
+        json.write(&mut buf, &array).unwrap();
         let json = String::from_utf8(buf).unwrap();
         assert_eq!(
             "[true,8,0.5,\"text\",{\"first\":null,\"second\":\"txt\"},[],{}]",
