@@ -1,6 +1,7 @@
 use std::fmt::Write;
 
 use crate::{
+    buffer::BufferHandle,
     buffer_position::{BufferPosition, BufferRange},
     buffer_view::{BufferViewHandle, CursorMovement, CursorMovementKind},
     client::ClientHandle,
@@ -9,21 +10,43 @@ use crate::{
     mode::{ModeKind, ModeState},
     platform::Key,
     plugin::{CompletionContext, PluginHandle},
-    word_database::{WordIndicesIter, WordKind},
+    word_database::WordKind,
 };
 
 #[derive(Default)]
 pub struct State {
+    editing_buffer_handle: Option<BufferHandle>,
     completion_positions: Vec<BufferPosition>,
     completing_plugin_handle: Option<PluginHandle>,
 }
 
+impl State {
+    pub fn on_buffer_insert_text(&mut self, handle: BufferHandle, range: BufferRange) {
+        if self.editing_buffer_handle == Some(handle) {
+            for position in &mut self.completion_positions {
+                if *position != range.from {
+                    *position = position.insert(range);
+                }
+            }
+        }
+    }
+
+    pub fn on_buffer_delete_text(&mut self, handle: BufferHandle, range: BufferRange) {
+        if self.editing_buffer_handle == Some(handle) {
+            for position in &mut self.completion_positions {
+                *position = position.delete(range);
+            }
+        }
+    }
+}
+
 impl ModeState for State {
     fn on_enter(editor: &mut Editor) {
         cancel_completion(editor);
     }
 
     fn on_exit(editor: &mut Editor) {
+        editor.mode.insert_state.editing_buffer_handle = None;
         cancel_completion(editor);
     }
 
@@ -40,6 +63,9 @@ impl ModeState for State {
             }
         };
 
+        ctx.editor.mode.insert_state.editing_buffer_handle =
+            Some(ctx.editor.buffer_views.get(handle).buffer_handle);
+
         let key = keys.next(&ctx.editor.buffered_keys);
         let register = ctx.editor.registers.get_mut(AUTO_MACRO_REGISTER);
         let _ = write!(register, "{}", key);
@@ -324,24 +350,9 @@ fn update_completions(
         }
     };
 
-    match ctx.editor.mode.insert_state.completing_plugin_handle {
-        Some(_) => {
-            ctx.editor
-                .picker
-                .filter(WordIndicesIter::empty(), completion_filter);
-        }
-        None => {
-            ctx.editor
-                .picker
-                .filter(ctx.editor.word_database.word_indices(), completion_filter);
-            if ctx.editor.picker.cursor().is_none() {
-                ctx.editor.picker.move_cursor(0);
-            }
-            if ctx.editor.picker.len() == 1 {
-                ctx.editor.picker.clear();
-            }
-        }
-    }
+    ctx.editor
+        .picker
+        .filter_completion(ctx.editor.word_database.word_indices(), completion_filter);
 }
 
 fn apply_completion(
