@@ -164,6 +164,7 @@ fn run_server(config: ApplicationConfig, listener: UnixListener) {
 
     let mut events = Vec::new();
     let mut timeout = None;
+    let mut need_redraw = false;
 
     const CLIENTS_START_INDEX: usize = 1;
     const CLIENTS_LAST_INDEX: usize = CLIENTS_START_INDEX + MAX_CLIENT_COUNT - 1;
@@ -175,20 +176,24 @@ fn run_server(config: ApplicationConfig, listener: UnixListener) {
     let mut epoll_events = EpollEvents::new();
 
     loop {
+        let previous_timeout = timeout;
         let epoll_events = epoll.wait(&mut epoll_events, timeout);
-        if epoll_events.len() == 0 {
+        let epoll_events_len = epoll_events.len();
+        if epoll_events_len == 0 {
             match timeout {
                 Some(Duration::ZERO) => timeout = Some(SERVER_IDLE_DURATION),
                 Some(_) => {
                     events.push(PlatformEvent::Idle);
                     timeout = None;
                 }
-                None => unreachable!(),
+                None => continue,
             }
         } else {
             timeout = Some(Duration::ZERO);
         }
 
+        let mut empty_write_event_count = 0;
+
         for (event_read, event_write, event_index) in epoll_events {
             match event_index {
                 0 => match listener.accept() {
@@ -202,7 +207,7 @@ fn run_server(config: ApplicationConfig, listener: UnixListener) {
                                 epoll.add(
                                     connection.as_raw_fd(),
                                     (CLIENTS_START_INDEX + i) as _,
-                                    libc::EPOLLET as _,
+                                    (libc::EPOLLOUT | libc::EPOLLET) as _,
                                 );
                                 *c = Some(connection);
                                 let handle = ClientHandle(i as _);
@@ -235,7 +240,7 @@ fn run_server(config: ApplicationConfig, listener: UnixListener) {
                         }
                     }
                     if let Some(ref mut connection) = client_connections[index] {
-                        if event_write {
+                        if event_write && !client_write_queue[index].is_empty() {
                             let result = write_to_connection(
                                 connection,
                                 &mut application.ctx.platform.buf_pool,
@@ -248,6 +253,10 @@ fn run_server(config: ApplicationConfig, listener: UnixListener) {
                             }
                         }
                     }
+
+                    if !event_read && event_write && client_write_queue[index].is_empty() {
+                        empty_write_event_count += 1;
+                    }
                 }
                 PROCESSES_START_INDEX..=PROCESSES_LAST_INDEX => {
                     let index = event_index - PROCESSES_START_INDEX;
@@ -271,6 +280,16 @@ fn run_server(config: ApplicationConfig, listener: UnixListener) {
             }
         }
 
+        if empty_write_event_count > 0 && empty_write_event_count == epoll_events_len {
+            timeout = previous_timeout;
+            continue;
+        }
+
+        if events.is_empty() && !need_redraw {
+            continue;
+        }
+
+        need_redraw = false;
         application.update(events.drain(..));
         let mut requests = application.ctx.platform.requests.drain();
         while let Some(request) = requests.next() {
@@ -286,7 +305,10 @@ fn run_server(config: ApplicationConfig, listener: UnixListener) {
                     }
                     return;
                 }
-                PlatformRequest::Redraw => timeout = Some(Duration::ZERO),
+                PlatformRequest::Redraw => {
+                    need_redraw = true;
+                    timeout = Some(Duration::ZERO);
+                }
                 PlatformRequest::WriteToClient { handle, buf } => {
                     let index = handle.0 as usize;
                     match client_connections[index] {
